library Volume;

// Integer logic functions for
// ProfiLab Expert V3.0 or higher
// (www.abacom-online.de)
// Feedback to: Expert.Profi@gmx.de

uses
  Math,
  SysUtils,
  Windows,
  Classes,
  Controls,
  mmSystem,
  Dlg_Vol in 'Dlg_Vol.pas' {ConfigDlg},
  Unit1 in 'Unit1.pas' {Form1},
  Unit3 in 'Unit3.pas' {Form3};

{$R *.RES}

Const  Inputs = 3;  // number of inputs

       {DLL INPUTS PINS}

       VolLPin = 0; // Volume Left 0...100 (%)
       VolRPin = 1; // Volume Right 0...100 (%)
       MutePin = 2; // Mute (Low = muted)

       {USER}
       U_VolL  = 0;         //old Volume L
       U_VolR  = 1;         //old Volume R
       U_VolMute  = 2;      //old Mute
       U_Device  = 3;       //Combo Setting
       U_Destination  = 4;  //Combo Setting
       U_Connection  = 5;   //Combo Setting


Type TDLLParams = array[0..100] of extended;   //Type of ProfiLab DLL parameters
     PDLLParams = ^TDLLParams;                 // Pointer to ProfiLab DLL parameters

Var DLG: TForm1;


function NumInputs: Byte;
begin
  result:=Inputs; //Define number of component input pins
end;

Function InputName(Channel: Byte): ShortString; // Return name for each component input pin
begin
   case Channel of
   VolLPin: result:='L';
   VolRPin: result:='R';
   MutePin: result:='/MUT';
   end;
end;

Procedure SimStart(PInput,POutput,PUser: PDLLParams); //called when ProfiLab enters RUN mode
begin
   //Make sure values set on start
   PUser^[U_VolL]:=-1;
   PUser^[U_VolR]:=-1;
   if (PUser^[U_VolMute]<2.5) then PUser^[U_VolMute]:=5 else PUser^[U_VolMute]:=0;
end;

Procedure Calculate(PInput,POutput,PUser: PDLLParams); //called regularly from ProfiLab
var VolL, VolR: extended;
    wVolL, wVolR: Word;
    Mute, VolLChanged, VolRChanged, MuteChanged: Boolean;
begin
   if not assigned(Dlg) then DLG:=TForm1.Create(nil); //public for all components!

   //Check for new Left Volume (0...100%)
   VolL:=PInput^[VolLPin];
   if VolL<0 then VolL:=0;
   if VolL>100 then VolL:=100;
   wVolL:=round($FFFF*VolL/100);
   VolLChanged:=wVolL<>round(PUser^[U_VolL]);
   PUser^[U_VolL]:=wVolL;

   //Check for new Right Volume (0...100%)
   VolR:=PInput^[VolRPin];
   if VolR<0 then VolR:=0;
   if VolR>100 then VolR:=100;
   wVolR:=round($FFFF*VolR/100);
   VolRChanged:=wVolR<>round(PUser^[U_VolR]);
   PUser^[U_VolR]:=wVolR;

   //Check for new Mute status (Low = Muted)
   Mute:=PInput^[MutePin]<2.5;
   MuteChanged:=Mute<>(PUser^[U_VolMute]<2.5);
   if Mute then PUser^[U_VolMute]:=0 else PUser^[U_VolMute]:=5;

   if MuteChanged or VolLChanged or VolRChanged then
   begin
      //Select Mixer as configured
      Dlg.Mixer.MixerId:=round(PUser^[U_Device]);
      //Change value(s)
      Dlg.Mixer.SetVolume(round(PUser^[U_Destination]), round(PUser^[U_Connection])-1, wVolL, wVolR, Integer(Mute));
   end;

end;

Procedure Configure(PUser: PDLLParams);
begin
   if not assigned(Dlg) then DLG:=TForm1.Create(nil); //public for all components!
   With Dlg do
   begin

      DeviceCombo.itemIndex:=round(PUser^[U_Device]);
      DeviceCombo.OnChange(DeviceCombo);

      DestinationCombo.itemIndex:=round(PUser^[U_Destination]);
      DestinationCombo.OnChange(DestinationCombo);

      ConnectionCombo.itemIndex:=round(PUser^[U_Connection]);
      ConnectionCombo.OnChange(ConnectionCombo);

      if ShowModal=mrOK then
      begin
         PUser^[U_Device]:=DeviceCombo.itemIndex;
         PUser^[U_Destination]:=DestinationCombo.itemIndex;
         PUser^[U_Connection]:=ConnectionCombo.itemIndex;
      end;
      
   end;
end;


//export methods for ProfiLab
exports SimStart,
        NumInputs,
        InputName,
        Calculate,
        Configure;
begin
end.

